use std::sync::Arc;

use axum::extract::{Query, State};
use axum::Form;
use sea_orm::{ActiveModelTrait, ColumnTrait, Condition, EntityTrait, NotSet, PaginatorTrait, QueryFilter, QueryOrder, QuerySelect};
use sea_orm::ActiveValue::Set;

use crate::entity::{article, category, tag};
use crate::err::AppError;
use crate::form::ArticleForm;
use crate::handler::{get_conn, HtmlResponse, log_error, redirect, RedirectResponse, render};
use crate::param::ArticleParams;
use crate::Result;
use crate::state::AppState;
use crate::view;

pub async fn index(
    State(state): State<Arc<AppState>>,
    Query(params): Query<ArticleParams>,
) -> Result<HtmlResponse> {
    let handler_name = "article/index";
    let conn = get_conn(&state);

    // 构建条件: 所有未被删除的文章
    let condition = Condition::all()
        .add(article::Column::IsDel.eq(false))
        .add_option(
            params.keyword_opt().map(|n| article::Column::Title.contains(&n))
        )
        .add_option(params.is_del_opt().map(|n| article::Column::IsDel.eq(n)));

    let mut select = article::Entity::find()
        .filter(condition);

    let page = params.page(); // 当前页码
    let page_size = params.page_size(); // 每页条数，默认15

    if let Some(ord) = params.order() {
        select = select.order_by(category::Column::Id, ord);
    }

    let paginator = select
        .find_also_related(category::Entity)
        .paginate(conn, page_size);

    let page_total = paginator.num_pages().await.map_err(AppError::from)?;

    let list: Vec<(article::Model, Option<category::Model>)> = paginator
        .fetch_page(page)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    let tpl = view::ArticlesTemplate {
        list,
        page_total,
        params,
    };
    render(tpl, handler_name)
}

pub async fn add_ui(State(state): State<Arc<AppState>>) -> Result<HtmlResponse> {
    let handler_name = "article/add_ui";
    let conn = get_conn(&state);

    let categies = category::Entity::find()
        .filter(category::Column::IsDel.eq(false))
        .limit(100)
        .order_by_asc(category::Column::Id)
        .all(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    let tpl = view::ArticleAddTemplate { categies };
    render(tpl, handler_name)
}

pub async fn add(
    State(state): State<Arc<AppState>>,
    Form(frm): Form<ArticleForm>,
) -> Result<RedirectResponse> {
    let handler_name = "article/add";
    let conn = get_conn(&state);

    article::ActiveModel {
        id: NotSet,
        title: Set(frm.title),
        category_id: Set(frm.category_id),
        content: Set(frm.content),
        ..Default::default()
    }.save(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    redirect("/article?msg=文章添加成功")
}

pub async fn list_with_tags( State(state): State<Arc<AppState>>) -> Result<String>  {
    let handler_name = "article/list_with_tags";
    let conn = get_conn(&state);

    let list = article::Entity::find()
        .find_with_related(tag::Entity)
        .all(conn)
        .await
        .map_err(AppError::from)
        .map_err(log_error(handler_name))?;

    let mut ss = vec![];

    for item in list {
        let (article, tags) = item;
        let tags = tags
            .iter()
            .map(|tag| format!("【#{} - {}】", &tag.id, &tag.name))
            .collect::<Vec<String>>()
            .join(",")
            .to_string();

        let s = format!(
            "文章ID: {}, 文章标题: {}, 标签： {}",
            &article.id, &article.title, tags,
        );
        ss.push(s);
    }

    Ok(ss.join("\n").to_string())
}